(* File absyn.ml *)
(* Stephanie Weirich, adapted from Amr Sabry *)

type var = string

type prim = Plus    (* + *)
          | Times   (* * *)
          | Minus   (* - *)

type exp = Var    of var             (* <var> *)
			| Lambda of var * exp       (* (lambda (<var>) <exp>) *)
			| App    of exp * exp       (* (<exp> <exp>) *)
			| Num    of int             (* <number> *)
			| Prim   of prim * exp list (* (<prim> {<exp>}) *)
			| Let    of var * exp * exp (* (let (<var> <exp>) <exp>) *)
				 
(********************************************************************)
(* Pretty-printing. Ignore... *)

let rec print_list 
  (exps:'a list)      (* what to print *)
  (pr:'a -> unit)     (* how to print each element *)
  (mid: unit -> unit) (* what to do between each element *)
  : unit = 
  match exps with 
		[] -> ()
	 | [last] -> pr last
	 | (hd :: tl) -> (pr hd;
							mid ();
							print_list tl pr mid)

let rec print_helper (e:exp) : unit = 
  match e with 
    Var s -> Format.print_string s
  | Lambda (s,e) -> 
      Format.open_box 2;
      Format.print_string "(lambda (";
      Format.print_string s;
      Format.print_string ")";
      Format.print_space();
      print_helper e;
      Format.print_string ")";
      Format.close_box()
  | App (e1,e2) -> 
      Format.open_box 2;
      Format.print_string "(";
      print_helper e1;
      Format.print_space();
      print_helper e2;
      Format.print_string ")";
      Format.close_box()
  | Num i -> 
		Format.print_int i
  | Prim (p,exps) -> 
		Format.open_box 2;
		Format.print_string "(";
		let prim_string = (match p with 
									  Plus -> "+"
									| Times -> "*"
									| Minus -> "-") in 
		Format.print_string prim_string;
		Format.print_space();
		print_list exps print_helper Format.print_space;
		Format.print_string ")";
		Format.close_box()
  | Let (v, exp1, exp2) -> 
		Format.open_box 2;
		Format.print_string "(let ";
		Format.open_box 2;
		Format.print_string "(";
		Format.print_string v;
		Format.print_space();
		print_helper exp1;
		Format.print_string ")";
		Format.close_box();
		Format.force_newline();
		print_helper exp2;
		Format.print_string ")";
		Format.close_box()	

(* A function to display an exp on the screen.
   Because its operation is entirely a side effect, the 
   result of this function is unit. *)
let print_exp (exp:exp) : unit = 
  Format.open_hbox();
  print_helper exp;
  Format.close_box();
  Format.print_newline()

(********************************************************************)

					 
									  
