(* File main.ml *)
(* Stephanie Weirich, adapted from Amr Sabry *)

let _ = 
  try 
    flush stdout;
    let lexbuf = Lexing.from_channel stdin in

    while true do
      let f _ : Absyn.exp option = 
        try
          let e = Mparser.main Mlexer.token lexbuf in
 				print_string "Input expression: ";
				print_newline();
				
				Absyn.print_exp e;
				
				let fvs = Scope.free_vars e in
				let bvs = Scope.bound_vars e in
				let value = Scope.eval e in 

				  print_string "Free variables: ";
				  Scope.print_vars fvs;
				  print_string "Bound variables: ";
				  Scope.print_vars bvs;
				  print_string "Reduces to: ";
				  Absyn.print_exp value;

				  print_string "Alpha-equivalent to itself:";
				  if (Scope.is_aeq e e) 
				  then print_string "true" 
				  else print_string "false";
				  print_string "\n******************\n";

				  Some e
        with 
				Parsing.Parse_error ->
              (Mlexer.print_error "Parse error\n"; None)
			 | Scope.TODO -> 
              (print_string "Not done\n"; None)
			 | Scope.RuntimeError s -> 
				  (print_string s; print_newline (); None)
      in
      
		let oe1 = f () in 
		let oe2 = f () in 
		 
      (match oe1,oe2 with 
		  Some e1,Some e2 -> 
			 print_string "\nAlpha-equivalent pairs:";
  			 if (Scope.is_aeq e1 e2) then print_string "true" else print_string "false";
			 print_string "\n******************\n";
		 |  _,_ -> ());
      flush stdout
    done
  with Mlexer.Eof -> exit 0


